/*****************************************************************************
*                                                                            *
* void      Create_HP_File(int Grautoene)                                    *
*                                                                            *
******************************************************************************
*                                                                            *
* Erzeugt eine Datei mit Steuerbefehlen fuer den HP-Laserjet und einer       *
* Bit-Matrix des Bildes. Graustufen werden in Grautonmuster umgesetzt.       *
*                                                                            *
******************************************************************************
*                                                                            *
* Erstellungsdatum: 19.08.1992                                               *
* Letzte Aenderung: 03.09.1992                                               *
*                                                                            *
******************************************************************************
*                                                                            *
* Input: Grautoene = 2, 5 oder 10                                            *
*                                                                            *
******************************************************************************
*                                                                            *
* Output:                                                                    *
*                                                                            *
******************************************************************************
*                                                                            *
* Aufruf von: WriteText(), ShowError(), Read_Int()                           *
*                                                                            *
*****************************************************************************/

#include "const.h"

#include "prototyp.h"


void Create_HP_File(int Grautoene)
{
  char HP_Name[80];                      /* File-Name ggf. mit PFad-Angabe  */
  char *Ext =".hp";                      /* Files haben immer Extension .hp */
  char EscSchluss[6];
  char Drehung, Invers;
  int  Resolution;
  int  PunkteProZeile, BytesProZeile;
  char ZeilenHeader[10];
  int  Dummy;
  int  Einer, Zehner, Hunderter;
  int  Columns, Lines, Hell, Dunkel;
  int  dx, dy, px, py;
  int HP_fp, AnzZeichen;
  long Index;

  int GrauTabelle[MAX_COLOR];
  unsigned long BitMask[3];
  /* Bit-Kombinationen bei Grautonmuster mit 5 oder 10 Graustufen           */
  static int GrautonM10[10][3]={{7,7,7},{5,7,7},{5,7,3},{4,7,3},{4,7,2},
                                {0,7,2},{0,3,2},{0,3,0},{0,2,0},{0,0,0}};
  static int GrautonM5 [ 5][2]={{3,3},{3,1},{2,1},{1,0},{0,0}};

  int Bit_Image[10][3];

  /* maximal drei komplette Punkt-Ausgabe-Zeilen werden zur Berechnung ben. */
  unsigned char OutZeile1[260],OutZeile2[260],OutZeile3[260];

  int i, j, z, SpaltenPointer, SpaltenZahl;
  int p;

  unsigned char OutChar, Maske;


  Read_String("\nName des HP-Files: ", HP_Name);

  if (!Add_Extension(HP_Name, Ext)) {
     WriteText( NO_CURS_POS, NO_CURS_POS, "\nFehler im File-Namen.");
     return;
  }

  if ((HP_fp = open(HP_Name, O_BINARY | O_CREAT | O_TRUNC | O_RDWR, S_IREAD | S_IWRITE)) < 0) {
     ShowError("Fehler beim Erzeugen des Files.");
     return;
  }

  WriteText( NO_CURS_POS, NO_CURS_POS, "\nAufloesungen der HP-Laserjets: 75, 150, 300 dpi\n");
  Resolution=Read_Int("Welche Aufloesung ", 300,' ');

  Drehung=Upper(Read_Char("Soll das Bild um 90 Grad gedreht werden     ", 'N'));

  Invers =Upper(Read_Char("Soll das Bild invertiert ausgedruckt werden ", 'N'));

  WriteText( NO_CURS_POS, NO_CURS_POS, "\nBeginn HP-File schreiben. ");

  /*------------------------ Header fuer HP-File aufbauen ------------------*/

  PunkteProZeile = (Drehung=='J') ? Bild[ZIEL_PIC].Zeilen:Bild[ZIEL_PIC].Spalten;

  if (Grautoene== 5) PunkteProZeile=2*PunkteProZeile;
  if (Grautoene==10) PunkteProZeile=3*PunkteProZeile;

  BytesProZeile = PunkteProZeile/8;
  if ((PunkteProZeile % 8) != 0) BytesProZeile++;

  EscSchluss[0]=ESC;         /* Schluss-Escape-Sequenz mit Form Feed        */
  EscSchluss[1]='*';
  EscSchluss[2]='r';
  EscSchluss[3]='B';
  EscSchluss[4]=FF;
  EscSchluss[5]='\0';

  ZeilenHeader[0]=ESC;       /* Transfer Raster-Graphik; Anzahl Bytes/Zeile */
  ZeilenHeader[1]='*';
  ZeilenHeader[2]='b';

  Dummy=BytesProZeile;
  Einer =Dummy % 10; Dummy/=10;
  Zehner=Dummy % 10;
  Hunderter= Dummy/10;

  if (Hunderter==0) {
     ZeilenHeader[3]=Zehner+'0';
     ZeilenHeader[4]=Einer +'0';
     ZeilenHeader[5]='W';
     ZeilenHeader[6]='\0';
  }
  else {
     ZeilenHeader[3]=Hunderter+'0';
     ZeilenHeader[4]=Zehner   +'0';
     ZeilenHeader[5]=Einer    +'0';
     ZeilenHeader[6]='W';
     ZeilenHeader[7]='\0';
  }
  AnzZeichen=strlen(ZeilenHeader);

  switch (Resolution) {
     case 75:                                 /* Raster-Graphik mit  75 dpi */
         write(HP_fp, EscSchluss, 1); write(HP_fp, "*t75R", 5);
         break;
     case 150:                                /* Raster-Graphik mit 150 dpi */
         write(HP_fp, EscSchluss, 1); write(HP_fp, "*t150R", 6);
         break;
     case 300:                                /* Raster-Graphik mit 300 dpi */
          write(HP_fp, EscSchluss, 1); write(HP_fp, "*t300R", 6);
  }

  if (Invers == 'J') { /* Falls inverse Ausgabe, hell u. dunkel vertauschen */
     Hell  =0;
     Dunkel=1;
  }
  else {
     Hell  =1;
     Dunkel=0;
  }

  /* Bild immer auf dem Blatt in der Mitte positionieren                    */
  Columns = (int)(DIN_A4_H * 300);   /* Anzahl der Spalten des Blattes      */
  Lines   = (int)(DIN_A4_V * 300);   /* Anzahl der Linien  des Blattes      */

  switch (Grautoene) {               /* Bildgroesse nimmt mit Grautoenen zu */
     case 2:
         px=Bild[ZIEL_PIC].Spalten;
         py=Bild[ZIEL_PIC].Zeilen;
         break;
     case 5:
         px=2*Bild[ZIEL_PIC].Spalten;
         py=2*Bild[ZIEL_PIC].Zeilen;
         break;
     case 10:
         px=3*Bild[ZIEL_PIC].Spalten;
         py=3*Bild[ZIEL_PIC].Zeilen;
  }

  if (Drehung == 'N') {
     dx = (Columns - ((long)px*300)/Resolution)/2;           /* freier Rand */
     dy = (Lines   - ((long)py*300)/Resolution)/2;
  }
  else {
     dx = (Columns - ((long)py*300)/Resolution)/2;
     dy = (Lines   - ((long)px*300)/Resolution)/2;
  }

  write(HP_fp, EscSchluss, 1);

  /* Linke obere Ecke im PCL-Koordinatensystem festlegen                    */
  sprintf(OutZeile1, "*p%dx%dY", dx, dy);
  write(HP_fp, OutZeile1, strlen(OutZeile1));

  write(HP_fp, EscSchluss, 1);
  write(HP_fp, "*r1A", 4);

  SpaltenZahl = Bild[ZIEL_PIC].Spalten;
  Index=PIC_SIZE;

  /*------------------------------------------------------------------------*/

  if (Grautoene == 2) {
     if (Drehung == 'N') {
        for (z=0; z<Bild[ZIEL_PIC].Zeilen; z++) {
            SpaltenPointer=0;
            for (i=0; i<BytesProZeile; i++) {
                OutChar=0;
                Maske = 0x80;                        /* Bit-Muster 10000000 */
                while ((Maske>0) && (SpaltenPointer<SpaltenZahl)) {
                   /* Bit==1 d.h. diesen Bildpunkt drucken                  */
                   OutChar += Maske*((Pictures[Index++]==SCHWARZ) ? Hell:Dunkel);
                   Maske = Maske>>1;
                   SpaltenPointer++;
                }
                OutZeile1[i]=OutChar;
            }
            write(HP_fp, ZeilenHeader, AnzZeichen);
            write(HP_fp, OutZeile1, BytesProZeile);
        }
     }

     else {                                 /* Bild wird um 90 Grad gedreht */

        for (z=0; z<Bild[ZIEL_PIC].Spalten; z++) {
            SpaltenPointer=Bild[ZIEL_PIC].Zeilen - 1;
            for (i=0; i<BytesProZeile; i++) {
                OutChar=0;
                Maske = 0x80;                        /* Bit-Muster 10000000 */
                while ((Maske>0) && (SpaltenPointer>=0)) {
                   /* Bit==1 d.h. diesen Bildpunkt drucken                  */
                   OutChar += Maske*((Pictures[Index+(long)(SpaltenPointer*SpaltenZahl)+z]==SCHWARZ) ? Hell:Dunkel);
                   Maske = Maske>>1;
                   SpaltenPointer--;
                }
                OutZeile1[i]=OutChar;
            }
            write(HP_fp, ZeilenHeader, AnzZeichen);
            write(HP_fp, OutZeile1, BytesProZeile);
        }
     }
  }

  /*------------------------------------------------------------------------*/

  if (Grautoene == 5) {

     for (i=0; i<Grautoene; i++)
        if (Hell==1) {
           Bit_Image[i][0]=GrautonM5[i][0];
           Bit_Image[i][1]=GrautonM5[i][1];
        }
        else {
           Bit_Image[i][0]=GrautonM5[4-i][0];
           Bit_Image[i][1]=GrautonM5[4-i][1];
     }

     for (i=0; i<MAX_COLOR; i++){
         if      (i>200) GrauTabelle[i]=4;
         else if (i>150) GrauTabelle[i]=3;
         else if (i>100) GrauTabelle[i]=2;
         else if (i> 50) GrauTabelle[i]=1;
         else            GrauTabelle[i]=0;
     }

     if (Drehung == 'N') {
        for (z=0; z<Bild[ZIEL_PIC].Zeilen; z++) {
            i=0;
            SpaltenPointer=0;
            while (SpaltenPointer<SpaltenZahl) {
                BitMask[0]=0; BitMask[1]=0;
                for (j=0; j<4; j++) {
                    p = ((SpaltenPointer++)<SpaltenZahl) ? Pictures[Index++]:WEISS;
                    BitMask[0]<<=2; BitMask[1]<<=2;
                    BitMask[0] |= Bit_Image[GrauTabelle[p]][0];
                    BitMask[1] |= Bit_Image[GrauTabelle[p]][1];
                }
                OutZeile1[i  ]=(BitMask[0] & 0xFF);
                OutZeile2[i++]=(BitMask[1] & 0xFF);
            }
  
            write(HP_fp, ZeilenHeader, AnzZeichen);
            write(HP_fp, OutZeile1, BytesProZeile);
            write(HP_fp, ZeilenHeader, AnzZeichen);
            write(HP_fp, OutZeile2, BytesProZeile);
        }
     }

     else {                                 /* Bild wird um 90 Grad gedreht */

        for (z=(SpaltenZahl-1); z>=0; z--) {
            SpaltenPointer=0; i=0; Index=z;
            while (SpaltenPointer<Bild[ZIEL_PIC].Zeilen) {
               BitMask[0]=0; BitMask[1]=0;
               for (j=0; j<4; j++) {
                   p=((SpaltenPointer++)<Bild[ZIEL_PIC].Zeilen) ? Pictures[Index]:241;
                   Index+=SpaltenZahl;
                   BitMask[0]<<=2; BitMask[1]<<=2; BitMask[2]<<=2;
                   BitMask[0] |= Bit_Image[GrauTabelle[p]][0];
                   BitMask[1] |= Bit_Image[GrauTabelle[p]][1];
               }
               OutZeile1[i  ]=(BitMask[0] & 0xFF);
               OutZeile2[i++]=(BitMask[1] & 0xFF);
            } /* while SpaltenPointer*/
  
            write(HP_fp, ZeilenHeader, AnzZeichen);
            write(HP_fp, OutZeile1, BytesProZeile);
            write(HP_fp, ZeilenHeader, AnzZeichen);
            write(HP_fp, OutZeile2, BytesProZeile);
        }
     }
  }

  /*------------------------------------------------------------------------*/

  if (Grautoene == 10) {

     for (i=0; i<10; i++)
        if (Hell==1) {
           Bit_Image[i][0]=GrautonM10[i][0];
           Bit_Image[i][1]=GrautonM10[i][1];
           Bit_Image[i][2]=GrautonM10[i][2];
        }
        else {
           Bit_Image[i][0]=GrautonM10[9-i][0];
           Bit_Image[i][1]=GrautonM10[9-i][1];
           Bit_Image[i][2]=GrautonM10[9-i][2];
     }

     for (i=0; i<MAX_COLOR; i++){
         if      (i>229) GrauTabelle[i]=9;
         else if (i>204) GrauTabelle[i]=8;
         else if (i>178) GrauTabelle[i]=7;
         else if (i>153) GrauTabelle[i]=6;
         else if (i>127) GrauTabelle[i]=5;
         else if (i>102) GrauTabelle[i]=4;
         else if (i> 76) GrauTabelle[i]=3;
         else if (i> 51) GrauTabelle[i]=2;
         else if (i> 25) GrauTabelle[i]=1;
         else            GrauTabelle[i]=0;
     }

     if (Drehung == 'N') {
        for (z=0; z<Bild[ZIEL_PIC].Zeilen; z++) {
            i=0;
            SpaltenPointer=0;
            while (SpaltenPointer<SpaltenZahl) {
                BitMask[0]=0; BitMask[1]=0; BitMask[2]=0;
                for (j=0; j<8; j++) {
                    p = ((SpaltenPointer++)<SpaltenZahl) ? Pictures[Index++]:WEISS;
                    BitMask[0]<<=3; BitMask[1]<<=3; BitMask[2]<<=3;
                    BitMask[0] |= Bit_Image[GrauTabelle[p]][0];
                    BitMask[1] |= Bit_Image[GrauTabelle[p]][1];
                    BitMask[2] |= Bit_Image[GrauTabelle[p]][2];
                }
                OutZeile1[i+2]=(BitMask[0] & 0xFF); BitMask[0]>>=8;
                OutZeile1[i+1]=(BitMask[0] & 0xFF); BitMask[0]>>=8;
                OutZeile1[i  ]=(BitMask[0] & 0xFF);

                OutZeile2[i+2]=(BitMask[1] & 0xFF); BitMask[1]>>=8;
                OutZeile2[i+1]=(BitMask[1] & 0xFF); BitMask[1]>>=8;
                OutZeile2[i  ]=(BitMask[1] & 0xFF);

                OutZeile3[i+2]=(BitMask[2] & 0xFF); BitMask[2]>>=8;
                OutZeile3[i+1]=(BitMask[2] & 0xFF); BitMask[2]>>=8;
                OutZeile3[i  ]=(BitMask[2] & 0xFF);
     
               i+=3;
            }
  
            write(HP_fp, ZeilenHeader, AnzZeichen);
            write(HP_fp, OutZeile1, BytesProZeile);
            write(HP_fp, ZeilenHeader, AnzZeichen);
            write(HP_fp, OutZeile2, BytesProZeile);
            write(HP_fp, ZeilenHeader, AnzZeichen);
            write(HP_fp, OutZeile3, BytesProZeile);
        }
     }

     else {                                 /* Bild wird um 90 Grad gedreht */

        for (z=(SpaltenZahl-1); z>=0; z--) {
            SpaltenPointer=0; i=0; Index=z;
            while (SpaltenPointer<Bild[ZIEL_PIC].Zeilen) {
               BitMask[0]=0; BitMask[1]=0; BitMask[2]=0;
               for (j=0; j<8; j++) {
                   p=((SpaltenPointer++)<Bild[ZIEL_PIC].Zeilen) ? Pictures[Index]:241;
                   Index+=SpaltenZahl;
                   BitMask[0]<<=3; BitMask[1]<<=3; BitMask[2]<<=3;
                   BitMask[0] |= Bit_Image[GrauTabelle[p]][0];
                   BitMask[1] |= Bit_Image[GrauTabelle[p]][1];
                   BitMask[2] |= Bit_Image[GrauTabelle[p]][2];
               }
               OutZeile1[i+2]=(BitMask[0] & 0xFF); BitMask[0]>>=8;
               OutZeile1[i+1]=(BitMask[0] & 0xFF); BitMask[0]>>=8;
               OutZeile1[i  ]=(BitMask[0] & 0xFF);

               OutZeile2[i+2]=(BitMask[1] & 0xFF); BitMask[1]>>=8;
               OutZeile2[i+1]=(BitMask[1] & 0xFF); BitMask[1]>>=8;
               OutZeile2[i  ]=(BitMask[1] & 0xFF);

               OutZeile3[i+2]=(BitMask[2] & 0xFF); BitMask[2]>>=8;
               OutZeile3[i+1]=(BitMask[2] & 0xFF); BitMask[2]>>=8;
               OutZeile3[i  ]=(BitMask[2] & 0xFF);
     
               i+=3;
            }
  
            write(HP_fp, ZeilenHeader, AnzZeichen);
            write(HP_fp, OutZeile1, BytesProZeile);
            write(HP_fp, ZeilenHeader, AnzZeichen);
            write(HP_fp, OutZeile2, BytesProZeile);
            write(HP_fp, ZeilenHeader, AnzZeichen);
            write(HP_fp, OutZeile3, BytesProZeile);
        }
     }
  }

  /*-------------------- Datei-Abschluss herausschreiben -------------------*/

  write(HP_fp, EscSchluss, 6);
  close(HP_fp);

  WriteText( NO_CURS_POS, NO_CURS_POS, "\nEnde HP-File schreiben.\n");

}
