/*****************************************************************************
*                                                                            *
* void      Ordered_Dith(int Verfahren)                                      *
*                                                                            *
******************************************************************************
*                                                                            *
* Erzeugt ein Binaerbild nach dem Ordered-Dither-Verfahren. Es werden drei   *
* unterschiedliche Matrizen benutzt.                                         *
*                                                                            *
******************************************************************************
*                                                                            *
* Erstellungsdatum: 24.08.1992                                               *
* Letzte Aenderung: 03.09.1992                                               *
*                                                                            *
******************************************************************************
*                                                                            *
* Input: Verfahren = 1 : Ordered-Dither mit 8x8-Matrix, aufloesungsoptimiert *
*        Verfahren = 2 : Ordered-Dither mit 8x8-Matrix, grauwertoptimiert    *
*        Verfahren = 3 : Ordered-Dither mit 4x4-Matrix                       *
*                                                                            *
******************************************************************************
*                                                                            *
* Output:                                                                    *
*                                                                            *
******************************************************************************
*                                                                            *
* Aufruf von: WriteText()                                                    *
*                                                                            *
*****************************************************************************/

#include "const.h"

#include "prototyp.h"


void Ordered_Dith(int Verfahren)
{
  int i, j, Klasse;
  int M_Groesse;                           /* Matrix-Groesse zum Dithern    */
  int ZeilenZahl, SpaltenZahl;
  long Index1, Index2;                     /* Adressen der einzelnen Bilder */

  int Muster[8][8];                        /* Hilfs-Array mit Dither-Matr.  */

  /*--------------------------- Dither-Matrizen ----------------------------*/
  /*
     8x8-Matrix - Optimiert bezgl. der Aufloesung
  static int Muster0[8][8]={{22, 6,18, 2,21, 5,17, 1},
                            {14,30,10,26,12,29, 9,25},
                            {20, 4,24, 8,19, 3,23, 7},
                            {12,28,16,32,11,27,15,31},
                            {21, 5,17, 1,22, 6,18, 2},
                            {13,29, 9,25,14,30,10,26},
                            {19, 3,23, 7,20, 4,24, 8},
                            {11,27,15,31,12,28,16,32}};
  */

  /* 8x8-Matrix - Optimiert bezgl. der Aufloesung */
  static int Muster1[8][8]={{ 0,32, 8,40, 2,43,10,42},
                            {48,16,56,24,50,18,58,26},
                            {12,44, 4,36,14,46, 6,38},
                            {60,28,52,20,62,30,54,22},
                            { 3,35,11,43, 1,33, 9,41},
                            {51,19,59,27,49,17,57,25},
                            {15,47, 7,39,13,45, 5,37},
                            {63,31,55,23,61,29,53,21}};

  /* 8x8-Matrix - Optimiert bezgl. den Graustufen */
  static int Muster2[8][8]={{18,24,22,16,13, 7, 9,15},
                            {20,30,28,26,11, 1, 3, 5},
                            {27,29,31,21, 4, 2, 0,10},
                            {17,23,25,19,14, 8, 6,12},
                            {13, 7, 9,15,18,24,22,16},
                            {11, 1, 3, 5,20,30,28,26},
                            { 4, 2, 0,10,27,29,31,21},
                            {14, 8, 6,12,17,23,25,19}};

  /* 4x4-Matrix - mehr graustufenorientiert als aufloesungorientiert */
  static int Muster3[4][4]={{ 5, 6, 7, 8},
                            { 4, 0, 1, 9},
                            {15, 3, 2,10},
                            {14,13,12,11}};
  
   
  WriteText(1, 25, "Bitte etwas Geduld ...");

  ZeilenZahl =Bild[QUELL_PIC].Zeilen;
  SpaltenZahl=Bild[QUELL_PIC].Spalten;
  Index1=0;
  Index2=PIC_SIZE;

  /* Initialisierung des Hilfs-Array mit den entsprechenden Werten          */
  if ((Verfahren==1) || (Verfahren==2)) {                 /* nur 8x8-Matrix */
     M_Groesse=8;
     for (i=0; i<M_Groesse; i++)
         for (j=0; j<M_Groesse; j++)
             Muster[i][j] = ((Verfahren==1) ? (4*Muster1[i][j]):(8*Muster2[i][j]));
  }
  else if (Verfahren==3) {                                /* 4x4-Matrix     */
     M_Groesse=4;
     for (i=0; i<M_Groesse; i++)
         for (j=0; j<M_Groesse; j++)
             Muster[i][j] = 16*Muster3[i][j];
  }
      
  /* den eigentlichen Dither-Vorgang mit Vergleich der Klassen starten      */
  for (i=0; i<ZeilenZahl; i++)
      for (j=0; j<SpaltenZahl; j++){
          Klasse=Muster[i % M_Groesse][j % M_Groesse];
          Pictures[Index2++] = (Pictures[Index1++]>Klasse) ? WEISS:SCHWARZ;
      }
  
  /* Groesse des Ergebnisbildes in der zugehoerigen Struktur ablegen        */
  Bild[ZIEL_PIC].Zeilen =Bild[QUELL_PIC].Zeilen;
  Bild[ZIEL_PIC].Spalten=Bild[QUELL_PIC].Spalten;

}
