/*****************************************************************************
*                                                                            *
****  Sammlung von Routinen zur I/O-, Fehlerbehandlung und zur Eingabe    ****
*                                                                            *
******************************************************************************
*                                                                            *
* int       Read_Int(char *String, int Default, char Buchstabe)              *
* void      Read_String(char Kommentar[], char String[])                     *
* char      Read_Char(char *String, char Value)                              *
* void      ShowError(char *Message)                                         *
* int       Add_Extension(char FileName[], char *Ext)                        *
* void      WriteText(int X_Pos, int Y_Pos, char *Text)                      *
* void      WriteInt(int Zahl)                                               *
* void      ClearTextWindow(int X_Left, int Y_Left, int X_Right, int Y_Right)*
*                                                                            *
*****************************************************************************/

#include "const.h"

#include "prototyp.h"


/*--------------------------------------------------------------------------*/
/* Liest fehlertolerant eine Int-Zahl ein. Ggf. wird ein Default-Wert vorg. */
int Read_Int(char *String, int Default, char Buchstabe)
{
  int  Tmp;
  char line[LINE_LENGTH];
  char Dummy[2];

  WriteText(NO_CURS_POS, NO_CURS_POS, String);
  if ( Default != INT_NO_DEFAULT ) {    /* falls Default-Wert angegeben,    */
     WriteText(NO_CURS_POS, NO_CURS_POS, "[");  /* diesen in [] ausgeben.   */
     WriteInt(Default);
     WriteText(NO_CURS_POS, NO_CURS_POS, "]");
  }
  *Dummy = Buchstabe;
  *(Dummy+1) = '\0';
  WriteText(NO_CURS_POS, NO_CURS_POS, Dummy);
  WriteText(NO_CURS_POS, NO_CURS_POS, " ");
  if ( !fgets(line, LINE_LENGTH, stdin) ) return( Default );
  if ( sscanf(line, "%d", &Tmp) != 1)     return( Default );

  return(Tmp);               /* ... ansonsten eingelesene Zahl zurueckgeben */
}


/*--------------------------------------------------------------------------*/
/* Liest einen String von der Tastatur ein. Vorher wird Kommentar ausgegeb. */
void Read_String(char Kommentar[], char String[])
{
  fprintf(stderr,"%s",Kommentar);
  fgets(String, LINE_LENGTH, stdin);

  String[strlen(String)-1]='\0';          /* String-Abschluss mit Null-Byte */
} 

/*--------------------------------------------------------------------------*/
/* Liest ein einzelnes Zeichen von der Tastatur ein                         */
/* String = Text-Meldung                                                    */
/* Value  = Default-Wert, der mit RETURN uebernommen werden kann            */
char Read_Char(char *String, char Value)
{
  char Tmp;
  char line[LINE_LENGTH];

  fprintf(stderr,"%s [%c] ", String, Value);
  if (fgets(line, LINE_LENGTH, stdin)==NULL) return(Value);
  if (sscanf(line, "%c", &Tmp)!=1)           return(Value);

  if (Tmp != CR) return(Tmp);      /* falls das letzte Zeichen nicht RETURN */
  else           return(Value);
}
  
/*--------------------------------------------------------------------------*/
/* Zeigt eine (Fehler-) Meldung an und wartet auf eine Bestaetigung         */
void ShowError(char *Message)
{
  char Dummy[LINE_LENGTH];

  WriteText(1, 23, Message);                /* Fehler-Meldungstext ausgeben */

  /* Bestaetigung und Form von RETURN-Taste abwarten                        */
  Read_String("\nBitte mit <RETURN> bestaetigen", Dummy);
}


/*--------------------------------------------------------------------------*/
/* Verwandelt alle Kleinbuchstaben in Grossbuchstaben. Rest bleibt unveraen.*/
char Upper(char c)
{
  return( ((c>='a')&&(c<='z')) ? c-('a'-'A'):c );
} 


/*--------------------------------------------------------------------------*/
/* Falls an einem Dateinamen die Extension fehlt wird diese angehaengt.     */
int Add_Extension(char FileName[], char *Ext)
{
  if (strlen(FileName)==0)        return(0);          /* Ueberhaupt was da? */
  else if (strlen(FileName)>=strlen(Ext)) {           /* String lang genug? */
     if(strcmp(FileName+(strlen(FileName)-4),Ext) != 0) /* noch keine Exten.*/
       strcat(FileName,Ext);
  }
  else strcat(FileName,Ext);                          /* Extension anhaengen*/

  return(1);                                         /* Erfolg signalisieren*/
}

/*--------------------------------------------------------------------------*/
/* Gibt einen beliebigen Zeichenstring auf stderr aus.                      */
void WriteText(int X_Pos, int Y_Pos, char *Text)
{
  if ((X_Pos>NO_CURS_POS) && (Y_Pos>NO_CURS_POS)) gotoxy(X_Pos, Y_Pos);
  fprintf(stderr, "%s", Text);
}

/*--------------------------------------------------------------------------*/
/* Gibt eine Int-Zahl auf stderr mit minimaler Formatanpassung aus.         */
void WriteInt(int Zahl)
{
  if     ((Zahl>999) || (Zahl<0))      /* Format ueber Zahlengroesse suchen */
     fprintf(stderr, "%d", Zahl);
  else if (Zahl> 99)
     fprintf(stderr, "%3d", Zahl);
  else if (Zahl>  9)
     fprintf(stderr, "%2d", Zahl);
  else
     fprintf(stderr, "%1d", Zahl);
}

/*--------------------------------------------------------------------------*/
/* Loescht den angegebenen Bildschirmteil im Graphikmodus mit Leerzeichen   */
/* (X_Left, Y_Left)   = linke obere Ecke                                    */
/* (X_Right, Y_Right) = rechte untere Ecke des Fensters                     */
void ClearTextWindow(int X_Left, int Y_Left, int X_Right, int Y_Right)
{
  int i,j;

  for (i=Y_Left; i<=Y_Right; i++)
      for (j=X_Left; j<=X_Right; j++)
          WriteText(j, i, " ");

}
